% $$$   This codes builds the dynamic matrix A for small perturbations to
% $$$   the parabolic profile in the channel flow. This is
% $$$   done by discretizing the Orr-Sommerfeld/Squire equations
% $$$   by mean of chebychev collocation for one wave number pair (kx,kz).
% $$$     
% $$$   It makes use of the Matlab differentiation suite (here functions chebdif.m, cheb4c.m) 
% $$$   written by Weideman and Reddy.
% $$$ 
% $$$   The velocity perturbation is expressed in the wall-normal velocity, 
% $$$   wall normal vorticity form.
% $$$ 
% $$$   It then plots the spectrum of the operator (complex eigenvalues)
% $$$   you can view the corresponding eigenvector by clicking on the desired eigenvalue)
% $$$ 
% $$$   Code written by jerome hoepffner, PhD student at KTH, Sweden
% $$$   For question, bugs and comments, contact jerome@mech.kth.se

%%%% parameters
Re=1000;%reynolds number (based on channel half width)
N=60;%number of collocation points in wall normal direction
kx=0.5;%streamwise wave number
kz=1;%spanwise wave number

%%%% differentiation matrices
[yvecT,DM] = chebdif(N+2,2);
yvec=yvecT(2:end-1);

%%%% the velocity profile
U.u  = 1-yvec.^2;                            
U.P = -2*yvec;                              
U.PP= -2*ones(size(yvec));

% implement homogeneous boundary conditions 
D2=DM(2:N+1,2:N+1,2);

% fourth derivative with clamped conditions
[y,D4]=cheb4c(N+2);

%%%% laplacian
I=eye(N);
k2=kx^2+kz^2;
delta=(D2-k2*I);
delta2=(D4-2*k2*D2+k2*k2*I);

%%%% compute dynamic matrix
LOS  = i* kx*diag(U.u)  * delta -i* kx*diag(U.PP)  - delta2 / Re ;
LC    = -i*kz*diag(U.P) ;
LSQ   =-i* kx*diag(U.u) + delta/Re;
A = [-delta\LOS    zeros(N,N);  LC  LSQ ];

%%%% plot the eigenvalues and set callback
figure(1);
[U,S]=eig(A);S=diag(S);
com=['figure(2);subplot(1,2,1);plot(real(U(1:N,@)),yvec,''b'',imag(U(1:N,@)),yvec,''r'');title(''wall normal velocity'');subplot(1,2,2);plot(real(U(N+1:2*N,@)),yvec,''b'',imag(U(N+1:2*N,@)),yvec,''r'');title(''wall normal vorticity'');'];
for ind=1:2*N
  han=plot(S(ind),'+');
  hold on
  set(han,'buttondownfcn',replaceind(com,'@',ind));
end  
hold off
xlim([-1,0]);